from .structs import Pair

AAVE_BTC = Pair("AAVE_BTC", price_precision=7, quantity_precision=4)
AAVE_USDC = Pair("AAVE_USDC", price_precision=2, quantity_precision=4)
AAVE_USDT = Pair("AAVE_USDT", price_precision=3, quantity_precision=4)
ACA_BTC = Pair("ACA_BTC", price_precision=9, quantity_precision=2)
ACA_USDT = Pair("ACA_USDT", price_precision=4, quantity_precision=2)
ADA_BTC = Pair("ADA_BTC", price_precision=9, quantity_precision=0)
ADA_USDC = Pair("ADA_USDC", price_precision=5, quantity_precision=1)
ADA_USDT = Pair("ADA_USDT", price_precision=5, quantity_precision=1)
AERGO_BTC = Pair("AERGO_BTC", price_precision=10, quantity_precision=1)
AERGO_USDT = Pair("AERGO_USDT", price_precision=5, quantity_precision=1)
AGLD_USDT = Pair("AGLD_USDT", price_precision=4, quantity_precision=2)
AKT_USDT = Pair("AKT_USDT", price_precision=4, quantity_precision=2)
ALGO_BTC = Pair("ALGO_BTC", price_precision=8, quantity_precision=2)
ALGO_USDC = Pair("ALGO_USDC", price_precision=4, quantity_precision=2)
ALGO_USDT = Pair("ALGO_USDT", price_precision=4, quantity_precision=2)
ALICE_BTC = Pair("ALICE_BTC", price_precision=8, quantity_precision=3)
ALICE_USDT = Pair("ALICE_USDT", price_precision=3, quantity_precision=3)
ALI_BTC = Pair("ALI_BTC", price_precision=10, quantity_precision=1)
ALI_USDT = Pair("ALI_USDT", price_precision=6, quantity_precision=0)
ALPHA_USDT = Pair("ALPHA_USDT", price_precision=5, quantity_precision=1)
AMP_USDT = Pair("AMP_USDT", price_precision=5, quantity_precision=1)
ANKR_USDT = Pair("ANKR_USDT", price_precision=6, quantity_precision=1)
APE_BTC = Pair("APE_BTC", price_precision=8, quantity_precision=3)
APE_USDC = Pair("APE_USDC", price_precision=4, quantity_precision=2)
APE_USDT = Pair("APE_USDT", price_precision=4, quantity_precision=2)
API3_BTC = Pair("API3_BTC", price_precision=8, quantity_precision=3)
API3_USDC = Pair("API3_USDC", price_precision=4, quantity_precision=2)
API3_USDT = Pair("API3_USDT", price_precision=4, quantity_precision=2)
ARPA_USDC = Pair("ARPA_USDC", price_precision=5, quantity_precision=1)
AR_USDC = Pair("AR_USDC", price_precision=3, quantity_precision=3)
AR_USDT = Pair("AR_USDT", price_precision=3, quantity_precision=3)
ATOM_BTC = Pair("ATOM_BTC", price_precision=7, quantity_precision=2)
ATOM_CRO = Pair("ATOM_CRO", price_precision=2, quantity_precision=2)
ATOM_USDC = Pair("ATOM_USDC", price_precision=3, quantity_precision=3)
ATOM_USDT = Pair("ATOM_USDT", price_precision=3, quantity_precision=2)
AUDIO_BTC = Pair("AUDIO_BTC", price_precision=9, quantity_precision=2)
AUDIO_USDC = Pair("AUDIO_USDC", price_precision=4, quantity_precision=2)
AUDIO_USDT = Pair("AUDIO_USDT", price_precision=4, quantity_precision=2)
AURORA_USDT = Pair("AURORA_USDT", price_precision=4, quantity_precision=2)
AUTO_USDT = Pair("AUTO_USDT", price_precision=2, quantity_precision=4)
AVAX_BTC = Pair("AVAX_BTC", price_precision=7, quantity_precision=3)
AVAX_USDC = Pair("AVAX_USDC", price_precision=3, quantity_precision=3)
AVAX_USDT = Pair("AVAX_USDT", price_precision=3, quantity_precision=3)
AXS_BTC = Pair("AXS_BTC", price_precision=7, quantity_precision=3)
AXS_USDC = Pair("AXS_USDC", price_precision=3, quantity_precision=3)
AXS_USDT = Pair("AXS_USDT", price_precision=4, quantity_precision=2)
BADGER_USDT = Pair("BADGER_USDT", price_precision=3, quantity_precision=3)
BAL_BTC = Pair("BAL_BTC", price_precision=8, quantity_precision=3)
BAL_USDT = Pair("BAL_USDT", price_precision=3, quantity_precision=3)
BAND_USDT = Pair("BAND_USDT", price_precision=4, quantity_precision=2)
BAT_USDT = Pair("BAT_USDT", price_precision=4, quantity_precision=2)
BCH_BTC = Pair("BCH_BTC", price_precision=6, quantity_precision=3)
BCH_USDT = Pair("BCH_USDT", price_precision=2, quantity_precision=5)
BICO_USDT = Pair("BICO_USDT", price_precision=4, quantity_precision=2)
BIFI_USDT = Pair("BIFI_USDT", price_precision=2, quantity_precision=5)
BNT_USDT = Pair("BNT_USDT", price_precision=4, quantity_precision=3)
BOND_USDC = Pair("BOND_USDC", price_precision=3, quantity_precision=3)
BOND_USDT = Pair("BOND_USDT", price_precision=3, quantity_precision=3)
BOSON_USDT = Pair("BOSON_USDT", price_precision=4, quantity_precision=2)
BRZ_USDT = Pair("BRZ_USDT", price_precision=5, quantity_precision=1)
BTC_USDC = Pair("BTC_USDC", price_precision=2, quantity_precision=6)
BTC_USDT = Pair("BTC_USDT", price_precision=2, quantity_precision=6)
BUSD_USDT = Pair("BUSD_USDT", price_precision=4, quantity_precision=2)
C98_USDT = Pair("C98_USDT", price_precision=4, quantity_precision=2)
CELR_USDC = Pair("CELR_USDC", price_precision=6, quantity_precision=0)
CELR_USDT = Pair("CELR_USDT", price_precision=6, quantity_precision=1)
CHR_BTC = Pair("CHR_BTC", price_precision=10, quantity_precision=1)
CHR_USDT = Pair("CHR_USDT", price_precision=5, quantity_precision=1)
CHZ_BTC = Pair("CHZ_BTC", price_precision=10, quantity_precision=1)
CHZ_USDT = Pair("CHZ_USDT", price_precision=5, quantity_precision=2)
CKB_USDT = Pair("CKB_USDT", price_precision=6, quantity_precision=0)
COMP_BTC = Pair("COMP_BTC", price_precision=7, quantity_precision=4)
COMP_USDT = Pair("COMP_USDT", price_precision=2, quantity_precision=5)
COTI_BTC = Pair("COTI_BTC", price_precision=10, quantity_precision=1)
COTI_USDC = Pair("COTI_USDC", price_precision=5, quantity_precision=1)
COTI_USDT = Pair("COTI_USDT", price_precision=5, quantity_precision=1)
CQT_USDT = Pair("CQT_USDT", price_precision=4, quantity_precision=2)
CRO_BTC = Pair("CRO_BTC", price_precision=9, quantity_precision=2)
CRO_USDC = Pair("CRO_USDC", price_precision=5, quantity_precision=3)
CRO_USDT = Pair("CRO_USDT", price_precision=5, quantity_precision=3)
CRPT_USDT = Pair("CRPT_USDT", price_precision=5, quantity_precision=1)
CRV_BTC = Pair("CRV_BTC", price_precision=8, quantity_precision=2)
CRV_USDC = Pair("CRV_USDC", price_precision=4, quantity_precision=2)
CRV_USDT = Pair("CRV_USDT", price_precision=4, quantity_precision=4)
CSPR_BTC = Pair("CSPR_BTC", price_precision=10, quantity_precision=1)
CSPR_USDT = Pair("CSPR_USDT", price_precision=5, quantity_precision=1)
CTSI_USDT = Pair("CTSI_USDT", price_precision=5, quantity_precision=1)
CVX_USDC = Pair("CVX_USDC", price_precision=3, quantity_precision=3)
CVX_USDT = Pair("CVX_USDT", price_precision=3, quantity_precision=3)
DAI_USDC = Pair("DAI_USDC", price_precision=4, quantity_precision=2)
DAI_USDT = Pair("DAI_USDT", price_precision=4, quantity_precision=3)
DAR_BTC = Pair("DAR_BTC", price_precision=9, quantity_precision=2)
DAR_USDC = Pair("DAR_USDC", price_precision=5, quantity_precision=1)
DAR_USDT = Pair("DAR_USDT", price_precision=4, quantity_precision=2)
DERC_USDC = Pair("DERC_USDC", price_precision=4, quantity_precision=2)
DERC_USDT = Pair("DERC_USDT", price_precision=4, quantity_precision=2)
DGB_USDC = Pair("DGB_USDC", price_precision=6, quantity_precision=0)
DIA_USDT = Pair("DIA_USDT", price_precision=4, quantity_precision=2)
DOGE_BTC = Pair("DOGE_BTC", price_precision=10, quantity_precision=1)
DOGE_USDC = Pair("DOGE_USDC", price_precision=5, quantity_precision=1)
DOGE_USDT = Pair("DOGE_USDT", price_precision=5, quantity_precision=2)
DOT_BTC = Pair("DOT_BTC", price_precision=7, quantity_precision=4)
DOT_USDC = Pair("DOT_USDC", price_precision=3, quantity_precision=3)
DOT_USDT = Pair("DOT_USDT", price_precision=4, quantity_precision=3)
DYDX_BTC = Pair("DYDX_BTC", price_precision=8, quantity_precision=3)
DYDX_USDT = Pair("DYDX_USDT", price_precision=3, quantity_precision=3)
EFI_BTC = Pair("EFI_BTC", price_precision=9, quantity_precision=2)
EFI_USDT = Pair("EFI_USDT", price_precision=4, quantity_precision=2)
EGLD_BTC = Pair("EGLD_BTC", price_precision=7, quantity_precision=4)
EGLD_USDC = Pair("EGLD_USDC", price_precision=2, quantity_precision=4)
EGLD_USDT = Pair("EGLD_USDT", price_precision=4, quantity_precision=3)
ELON_USDC = Pair("ELON_USDC", price_precision=10, quantity_precision=0)
ELON_USDT = Pair("ELON_USDT", price_precision=10, quantity_precision=0)
ENJ_BTC = Pair("ENJ_BTC", price_precision=9, quantity_precision=2)
ENJ_USDC = Pair("ENJ_USDC", price_precision=4, quantity_precision=2)
ENJ_USDT = Pair("ENJ_USDT", price_precision=5, quantity_precision=1)
ENS_BTC = Pair("ENS_BTC", price_precision=8, quantity_precision=3)
ENS_USDC = Pair("ENS_USDC", price_precision=3, quantity_precision=3)
ENS_USDT = Pair("ENS_USDT", price_precision=3, quantity_precision=3)
EOS_BTC = Pair("EOS_BTC", price_precision=8, quantity_precision=2)
EOS_USDT = Pair("EOS_USDT", price_precision=4, quantity_precision=2)
ERN_USDT = Pair("ERN_USDT", price_precision=4, quantity_precision=2)
ETC_BTC = Pair("ETC_BTC", price_precision=7, quantity_precision=3)
ETC_USDT = Pair("ETC_USDT", price_precision=3, quantity_precision=3)
ETH_BTC = Pair("ETH_BTC", price_precision=6, quantity_precision=5)
ETH_CRO = Pair("ETH_CRO", price_precision=2, quantity_precision=5)
ETH_USDC = Pair("ETH_USDC", price_precision=2, quantity_precision=5)
ETH_USDT = Pair("ETH_USDT", price_precision=2, quantity_precision=5)
FARM_USDC = Pair("FARM_USDC", price_precision=2, quantity_precision=4)
FET_USDT = Pair("FET_USDT", price_precision=5, quantity_precision=1)
FIL_BTC = Pair("FIL_BTC", price_precision=7, quantity_precision=3)
FIL_USDT = Pair("FIL_USDT", price_precision=3, quantity_precision=3)
FITFI_BTC = Pair("FITFI_BTC", price_precision=9, quantity_precision=2)
FITFI_USDT = Pair("FITFI_USDT", price_precision=5, quantity_precision=1)
FLOW_BTC = Pair("FLOW_BTC", price_precision=8, quantity_precision=2)
FLOW_USDC = Pair("FLOW_USDC", price_precision=4, quantity_precision=2)
FLOW_USDT = Pair("FLOW_USDT", price_precision=3, quantity_precision=4)
FLUX_USDT = Pair("FLUX_USDT", price_precision=4, quantity_precision=2)
FORTH_USDT = Pair("FORTH_USDT", price_precision=3, quantity_precision=3)
FTM_BTC = Pair("FTM_BTC", price_precision=9, quantity_precision=2)
FTM_USDC = Pair("FTM_USDC", price_precision=4, quantity_precision=2)
FTM_USDT = Pair("FTM_USDT", price_precision=4, quantity_precision=2)
FXS_BTC = Pair("FXS_BTC", price_precision=8, quantity_precision=3)
FXS_USDC = Pair("FXS_USDC", price_precision=4, quantity_precision=2)
FXS_USDT = Pair("FXS_USDT", price_precision=3, quantity_precision=2)
GALA_BTC = Pair("GALA_BTC", price_precision=9, quantity_precision=1)
GALA_USDC = Pair("GALA_USDC", price_precision=5, quantity_precision=1)
GALA_USDT = Pair("GALA_USDT", price_precision=6, quantity_precision=0)
GHST_USDC = Pair("GHST_USDC", price_precision=4, quantity_precision=2)
GLMR_BTC = Pair("GLMR_BTC", price_precision=9, quantity_precision=2)
GLMR_USDT = Pair("GLMR_USDT", price_precision=4, quantity_precision=2)
GLM_USDT = Pair("GLM_USDT", price_precision=5, quantity_precision=1)
GODS_USDT = Pair("GODS_USDT", price_precision=4, quantity_precision=2)
GRT_BTC = Pair("GRT_BTC", price_precision=10, quantity_precision=1)
GRT_USDT = Pair("GRT_USDT", price_precision=5, quantity_precision=2)
GTC_USDT = Pair("GTC_USDT", price_precision=3, quantity_precision=3)
HBAR_BTC = Pair("HBAR_BTC", price_precision=10, quantity_precision=1)
HBAR_USDT = Pair("HBAR_USDT", price_precision=5, quantity_precision=1)
HERO_USDT = Pair("HERO_USDT", price_precision=6, quantity_precision=0)
HIGH_USDT = Pair("HIGH_USDT", price_precision=4, quantity_precision=2)
HNT_BTC = Pair("HNT_BTC", price_precision=8, quantity_precision=3)
HNT_USDC = Pair("HNT_USDC", price_precision=3, quantity_precision=3)
HNT_USDT = Pair("HNT_USDT", price_precision=3, quantity_precision=3)
HOD_USDT = Pair("HOD_USDT", price_precision=6, quantity_precision=0)
HOT_USDT = Pair("HOT_USDT", price_precision=6, quantity_precision=0)
ICP_BTC = Pair("ICP_BTC", price_precision=8, quantity_precision=3)
ICP_USDC = Pair("ICP_USDC", price_precision=3, quantity_precision=3)
ICP_USDT = Pair("ICP_USDT", price_precision=3, quantity_precision=3)
ICX_BTC = Pair("ICX_BTC", price_precision=8, quantity_precision=1)
ICX_USDT = Pair("ICX_USDT", price_precision=4, quantity_precision=2)
ILV_BTC = Pair("ILV_BTC", price_precision=6, quantity_precision=4)
ILV_USDT = Pair("ILV_USDT", price_precision=2, quantity_precision=4)
IMX_BTC = Pair("IMX_BTC", price_precision=9, quantity_precision=2)
IMX_USDC = Pair("IMX_USDC", price_precision=4, quantity_precision=2)
IMX_USDT = Pair("IMX_USDT", price_precision=4, quantity_precision=2)
INJ_USDC = Pair("INJ_USDC", price_precision=4, quantity_precision=2)
INJ_USDT = Pair("INJ_USDT", price_precision=3, quantity_precision=3)
IOTX_USDC = Pair("IOTX_USDC", price_precision=6, quantity_precision=0)
IOTX_USDT = Pair("IOTX_USDT", price_precision=5, quantity_precision=1)
IQ_USDT = Pair("IQ_USDT", price_precision=6, quantity_precision=0)
JASMY_BTC = Pair("JASMY_BTC", price_precision=10, quantity_precision=1)
JASMY_USDC = Pair("JASMY_USDC", price_precision=5, quantity_precision=1)
JASMY_USDT = Pair("JASMY_USDT", price_precision=5, quantity_precision=1)
JOE_USDT = Pair("JOE_USDT", price_precision=5, quantity_precision=1)
KAVA_USDT = Pair("KAVA_USDT", price_precision=4, quantity_precision=2)
KLAY_USDT = Pair("KLAY_USDT", price_precision=4, quantity_precision=2)
KNC_BTC = Pair("KNC_BTC", price_precision=9, quantity_precision=2)
KNC_USDT = Pair("KNC_USDT", price_precision=4, quantity_precision=3)
KP3R_USDT = Pair("KP3R_USDT", price_precision=2, quantity_precision=4)
KRL_USDT = Pair("KRL_USDT", price_precision=5, quantity_precision=1)
KSM_BTC = Pair("KSM_BTC", price_precision=7, quantity_precision=4)
KSM_USDC = Pair("KSM_USDC", price_precision=2, quantity_precision=4)
KSM_USDT = Pair("KSM_USDT", price_precision=3, quantity_precision=3)
LDO_BTC = Pair("LDO_BTC", price_precision=9, quantity_precision=2)
LDO_USDT = Pair("LDO_USDT", price_precision=4, quantity_precision=2)
LINK_BTC = Pair("LINK_BTC", price_precision=7, quantity_precision=2)
LINK_CRO = Pair("LINK_CRO", price_precision=2, quantity_precision=2)
LINK_USDC = Pair("LINK_USDC", price_precision=3, quantity_precision=3)
LINK_USDT = Pair("LINK_USDT", price_precision=3, quantity_precision=2)
LPT_USDC = Pair("LPT_USDC", price_precision=3, quantity_precision=3)
LRC_USDT = Pair("LRC_USDT", price_precision=5, quantity_precision=2)
LSK_USDT = Pair("LSK_USDT", price_precision=4, quantity_precision=2)
LTC_BTC = Pair("LTC_BTC", price_precision=6, quantity_precision=4)
LTC_USDC = Pair("LTC_USDC", price_precision=2, quantity_precision=4)
LTC_USDT = Pair("LTC_USDT", price_precision=2, quantity_precision=5)
MANA_BTC = Pair("MANA_BTC", price_precision=9, quantity_precision=1)
MANA_USDC = Pair("MANA_USDC", price_precision=4, quantity_precision=2)
MANA_USDT = Pair("MANA_USDT", price_precision=5, quantity_precision=2)
MASK_USDT = Pair("MASK_USDT", price_precision=4, quantity_precision=2)
MATIC_BTC = Pair("MATIC_BTC", price_precision=9, quantity_precision=0)
MATIC_USDC = Pair("MATIC_USDC", price_precision=5, quantity_precision=1)
MATIC_USDT = Pair("MATIC_USDT", price_precision=5, quantity_precision=2)
MBL_BTC = Pair("MBL_BTC", price_precision=10, quantity_precision=0)
MBL_USDT = Pair("MBL_USDT", price_precision=6, quantity_precision=0)
MC_USDT = Pair("MC_USDT", price_precision=4, quantity_precision=2)
MDT_USDT = Pair("MDT_USDT", price_precision=6, quantity_precision=0)
MKR_USDT = Pair("MKR_USDT", price_precision=2, quantity_precision=6)
MLN_USDC = Pair("MLN_USDC", price_precision=3, quantity_precision=3)
MMF_USDC = Pair("MMF_USDC", price_precision=4, quantity_precision=2)
MOVR_BTC = Pair("MOVR_BTC", price_precision=7, quantity_precision=4)
MOVR_USDT = Pair("MOVR_USDT", price_precision=2, quantity_precision=4)
MULTI_BTC = Pair("MULTI_BTC", price_precision=8, quantity_precision=3)
MULTI_USDT = Pair("MULTI_USDT", price_precision=3, quantity_precision=3)
MXC_USDT = Pair("MXC_USDT", price_precision=5, quantity_precision=1)
NEAR_BTC = Pair("NEAR_BTC", price_precision=8, quantity_precision=2)
NEAR_USDC = Pair("NEAR_USDC", price_precision=3, quantity_precision=3)
NEAR_USDT = Pair("NEAR_USDT", price_precision=4, quantity_precision=2)
NEO_BTC = Pair("NEO_BTC", price_precision=7, quantity_precision=3)
NEO_USDT = Pair("NEO_USDT", price_precision=3, quantity_precision=4)
NKN_USDT = Pair("NKN_USDT", price_precision=5, quantity_precision=1)
NMR_USDT = Pair("NMR_USDT", price_precision=3, quantity_precision=3)
OCEAN_USDT = Pair("OCEAN_USDT", price_precision=4, quantity_precision=2)
OGN_BTC = Pair("OGN_BTC", price_precision=9, quantity_precision=2)
OGN_USDT = Pair("OGN_USDT", price_precision=4, quantity_precision=2)
OMG_BTC = Pair("OMG_BTC", price_precision=8, quantity_precision=4)
OMG_USDT = Pair("OMG_USDT", price_precision=4, quantity_precision=4)
ONEINCH_BTC = Pair("1INCH_BTC", price_precision=9, quantity_precision=2)
ONEINCH_USDC = Pair("1INCH_USDC", price_precision=4, quantity_precision=2)
ONEINCH_USDT = Pair("1INCH_USDT", price_precision=4, quantity_precision=2)
ONE_BTC = Pair("ONE_BTC", price_precision=10, quantity_precision=1)
ONE_USDC = Pair("ONE_USDC", price_precision=5, quantity_precision=1)
ONE_USDT = Pair("ONE_USDT", price_precision=5, quantity_precision=1)
ONT_BTC = Pair("ONT_BTC", price_precision=8, quantity_precision=2)
ONT_USDT = Pair("ONT_USDT", price_precision=4, quantity_precision=2)
OXT_USDT = Pair("OXT_USDT", price_precision=5, quantity_precision=1)
PAXG_USDT = Pair("PAXG_USDT", price_precision=2, quantity_precision=5)
PENDLE_USDT = Pair("PENDLE_USDT", price_precision=5, quantity_precision=1)
PERP_USDT = Pair("PERP_USDT", price_precision=3, quantity_precision=3)
PLA_USDT = Pair("PLA_USDT", price_precision=5, quantity_precision=1)
POLS_USDT = Pair("POLS_USDT", price_precision=4, quantity_precision=2)
POLY_USDC = Pair("POLY_USDC", price_precision=5, quantity_precision=1)
POWR_USDT = Pair("POWR_USDT", price_precision=5, quantity_precision=1)
PRQ_BTC = Pair("PRQ_BTC", price_precision=10, quantity_precision=1)
PRQ_USDT = Pair("PRQ_USDT", price_precision=5, quantity_precision=1)
PYR_BTC = Pair("PYR_BTC", price_precision=8, quantity_precision=3)
PYR_USDC = Pair("PYR_USDC", price_precision=3, quantity_precision=3)
PYR_USDT = Pair("PYR_USDT", price_precision=3, quantity_precision=3)
QI_USDT = Pair("QI_USDT", price_precision=5, quantity_precision=1)
QNT_USDC = Pair("QNT_USDC", price_precision=2, quantity_precision=4)
QNT_USDT = Pair("QNT_USDT", price_precision=2, quantity_precision=4)
QRDO_USDT = Pair("QRDO_USDT", price_precision=4, quantity_precision=2)
QTUM_USDT = Pair("QTUM_USDT", price_precision=4, quantity_precision=4)
QUICK_USDT = Pair("QUICK_USDT", price_precision=2, quantity_precision=4)
RADAR_BTC = Pair("RADAR_BTC", price_precision=10, quantity_precision=0)
RADAR_USDT = Pair("RADAR_USDT", price_precision=6, quantity_precision=0)
RAD_USDC = Pair("RAD_USDC", price_precision=3, quantity_precision=3)
RARE_USDT = Pair("RARE_USDT", price_precision=5, quantity_precision=1)
RARI_USDC = Pair("RARI_USDC", price_precision=3, quantity_precision=3)
RARI_USDT = Pair("RARI_USDT", price_precision=3, quantity_precision=3)
RAY_USDT = Pair("RAY_USDT", price_precision=4, quantity_precision=2)
REEF_USDT = Pair("REEF_USDT", price_precision=7, quantity_precision=0)
REN_USDT = Pair("REN_USDT", price_precision=5, quantity_precision=2)
REQ_USDT = Pair("REQ_USDT", price_precision=5, quantity_precision=1)
RLC_USDT = Pair("RLC_USDT", price_precision=3, quantity_precision=3)
RLY_USDT = Pair("RLY_USDT", price_precision=5, quantity_precision=2)
RNDR_USDT = Pair("RNDR_USDT", price_precision=4, quantity_precision=2)
RSR_USDT = Pair("RSR_USDT", price_precision=5, quantity_precision=1)
RUNE_BTC = Pair("RUNE_BTC", price_precision=8, quantity_precision=3)
RUNE_USDT = Pair("RUNE_USDT", price_precision=3, quantity_precision=3)
SAND_BTC = Pair("SAND_BTC", price_precision=8, quantity_precision=2)
SAND_USDC = Pair("SAND_USDC", price_precision=4, quantity_precision=2)
SAND_USDT = Pair("SAND_USDT", price_precision=5, quantity_precision=2)
SC_USDT = Pair("SC_USDT", price_precision=6, quantity_precision=0)
SDN_USDC = Pair("SDN_USDC", price_precision=4, quantity_precision=2)
SDN_USDT = Pair("SDN_USDT", price_precision=4, quantity_precision=2)
SHIB_USDC = Pair("SHIB_USDC", price_precision=9, quantity_precision=0)
SHIB_USDT = Pair("SHIB_USDT", price_precision=9, quantity_precision=0)
SKL_BTC = Pair("SKL_BTC", price_precision=10, quantity_precision=1)
SKL_USDT = Pair("SKL_USDT", price_precision=5, quantity_precision=2)
SLP_BTC = Pair("SLP_BTC", price_precision=10, quantity_precision=0)
SLP_USDC = Pair("SLP_USDC", price_precision=6, quantity_precision=0)
SLP_USDT = Pair("SLP_USDT", price_precision=5, quantity_precision=1)
SNT_USDT = Pair("SNT_USDT", price_precision=6, quantity_precision=0)
SNX_USDT = Pair("SNX_USDT", price_precision=3, quantity_precision=3)
SOL_BTC = Pair("SOL_BTC", price_precision=7, quantity_precision=3)
SOL_USDC = Pair("SOL_USDC", price_precision=3, quantity_precision=3)
SOL_USDT = Pair("SOL_USDT", price_precision=3, quantity_precision=3)
SPELL_USDC = Pair("SPELL_USDC", price_precision=6, quantity_precision=0)
SPELL_USDT = Pair("SPELL_USDT", price_precision=7, quantity_precision=0)
SPS_USDT = Pair("SPS_USDT", price_precision=5, quantity_precision=1)
SRM_USDC = Pair("SRM_USDC", price_precision=4, quantity_precision=2)
SRM_USDT = Pair("SRM_USDT", price_precision=4, quantity_precision=2)
STORJ_USDT = Pair("STORJ_USDT", price_precision=4, quantity_precision=2)
STRAX_USDT = Pair("STRAX_USDT", price_precision=4, quantity_precision=2)
STX_BTC = Pair("STX_BTC", price_precision=9, quantity_precision=2)
STX_USDC = Pair("STX_USDC", price_precision=4, quantity_precision=2)
STX_USDT = Pair("STX_USDT", price_precision=4, quantity_precision=2)
SUPER_USDT = Pair("SUPER_USDT", price_precision=4, quantity_precision=2)
SUSHI_BTC = Pair("SUSHI_BTC", price_precision=8, quantity_precision=2)
SUSHI_USDT = Pair("SUSHI_USDT", price_precision=3, quantity_precision=3)
TFUEL_USDT = Pair("TFUEL_USDT", price_precision=5, quantity_precision=1)
THETA_USDC = Pair("THETA_USDC", price_precision=4, quantity_precision=2)
THETA_USDT = Pair("THETA_USDT", price_precision=4, quantity_precision=2)
TONIC_USDC = Pair("TONIC_USDC", price_precision=10, quantity_precision=0)
TONIC_USDT = Pair("TONIC_USDT", price_precision=10, quantity_precision=0)
TRB_USDT = Pair("TRB_USDT", price_precision=3, quantity_precision=3)
TRIBE_USDT = Pair("TRIBE_USDT", price_precision=4, quantity_precision=2)
TRU_USDT = Pair("TRU_USDT", price_precision=5, quantity_precision=1)
TUSD_USDT = Pair("TUSD_USDT", price_precision=5, quantity_precision=1)
T_USDT = Pair("T_USDT", price_precision=5, quantity_precision=1)
UMA_USDT = Pair("UMA_USDT", price_precision=3, quantity_precision=4)
UNI_BTC = Pair("UNI_BTC", price_precision=8, quantity_precision=3)
UNI_USDC = Pair("UNI_USDC", price_precision=3, quantity_precision=3)
UNI_USDT = Pair("UNI_USDT", price_precision=4, quantity_precision=4)
USDC_USDT = Pair("USDC_USDT", price_precision=4, quantity_precision=2)
VET_BTC = Pair("VET_BTC", price_precision=10, quantity_precision=0)
VET_USDC = Pair("VET_USDC", price_precision=6, quantity_precision=0)
VET_USDT = Pair("VET_USDT", price_precision=6, quantity_precision=0)
VOXEL_USDT = Pair("VOXEL_USDT", price_precision=4, quantity_precision=2)
VRA_USDT = Pair("VRA_USDT", price_precision=6, quantity_precision=0)
VTHO_USDT = Pair("VTHO_USDT", price_precision=6, quantity_precision=0)
VVS_USDC = Pair("VVS_USDC", price_precision=8, quantity_precision=0)
VVS_USDT = Pair("VVS_USDT", price_precision=8, quantity_precision=0)
WAVES_BTC = Pair("WAVES_BTC", price_precision=8, quantity_precision=3)
WAVES_USDT = Pair("WAVES_USDT", price_precision=3, quantity_precision=3)
WAXP_USDT = Pair("WAXP_USDT", price_precision=5, quantity_precision=1)
WBTC_BTC = Pair("WBTC_BTC", price_precision=4, quantity_precision=6)
WBTC_USDC = Pair("WBTC_USDC", price_precision=2, quantity_precision=6)
WBTC_USDT = Pair("WBTC_USDT", price_precision=2, quantity_precision=6)
WEMIX_USDT = Pair("WEMIX_USDT", price_precision=4, quantity_precision=2)
WOO_USDT = Pair("WOO_USDT", price_precision=5, quantity_precision=1)
XLM_BTC = Pair("XLM_BTC", price_precision=9, quantity_precision=0)
XLM_USDC = Pair("XLM_USDC", price_precision=5, quantity_precision=1)
XLM_USDT = Pair("XLM_USDT", price_precision=5, quantity_precision=1)
XRP_BTC = Pair("XRP_BTC", price_precision=9, quantity_precision=0)
XRP_USDC = Pair("XRP_USDC", price_precision=5, quantity_precision=1)
XRP_USDT = Pair("XRP_USDT", price_precision=5, quantity_precision=1)
XTZ_BTC = Pair("XTZ_BTC", price_precision=8, quantity_precision=2)
XTZ_USDT = Pair("XTZ_USDT", price_precision=4, quantity_precision=2)
XYO_USDC = Pair("XYO_USDC", price_precision=6, quantity_precision=0)
XYO_USDT = Pair("XYO_USDT", price_precision=6, quantity_precision=0)
YFI_BTC = Pair("YFI_BTC", price_precision=4, quantity_precision=6)
YFI_USDT = Pair("YFI_USDT", price_precision=2, quantity_precision=6)
YGG_BTC = Pair("YGG_BTC", price_precision=8, quantity_precision=2)
YGG_USDT = Pair("YGG_USDT", price_precision=4, quantity_precision=2)
ZBC_USDT = Pair("ZBC_USDT", price_precision=6, quantity_precision=0)
ZIL_BTC = Pair("ZIL_BTC", price_precision=10, quantity_precision=1)
ZIL_USDT = Pair("ZIL_USDT", price_precision=5, quantity_precision=1)
ZRX_USDT = Pair("ZRX_USDT", price_precision=4, quantity_precision=2)


def all():
    return [
        value for name, value in globals().items() if isinstance(value, Pair)
    ]



import asyncio
from typing import Dict, List

from .api import ApiError, ApiProvider
from .market import Exchange
from .structs import (
    Balance,
    Coin,
    Deposit,
    DepositStatus,
    Interest,
    Order,
    OrderExecType,
    OrderForceType,
    OrderSide,
    OrderStatus,
    OrderType,
    Pair,
    PrivateTrade,
    Withdrawal,
    WithdrawalStatus,
)


class Account:
    """Provides access to account actions and data. Balance, trades, orders."""

    def __init__(
        self,
        *,
        api_key: str = "",
        api_secret: str = "",
        from_env: bool = False,
        exchange: Exchange = None,
        api: ApiProvider = None,
    ):
        if not api and not (api_key and api_secret) and not from_env:
            raise ValueError(
                "Pass ApiProvider or api_key with api_secret or from_env"
            )
        self.api = api or ApiProvider(
            api_key=api_key, api_secret=api_secret, from_env=from_env
        )
        self.exchange = exchange or Exchange(api)
        self.pairs = self.exchange.pairs

    async def sync_pairs(self):
        await self.exchange.sync_pairs()
        self.pairs = self.exchange.pairs

    async def get_balance(self) -> Dict[Coin, Balance]:
        """Return balance."""
        data = await self.api.post("private/get-account-summary")
        return {
            Coin(bal["currency"]): Balance.from_api(bal)
            for bal in data["accounts"]
        }

    async def get_deposit_history(
        self,
        coin: Coin,
        start_ts: int = None,
        end_ts: int = None,
        status: DepositStatus = None,
        page: int = 0,
        page_size: int = 20,
    ) -> List[Deposit]:
        """Return all history withdrawals."""
        params = {"page_size": page_size, "page": page}
        if coin:
            params["currency"] = coin.name
        if start_ts:
            params["start_ts"] = int(start_ts) * 1000
        if end_ts:
            params["end_ts"] = int(end_ts) * 1000
        if status:
            params["status"] = status

        data = (
            await self.api.post(
                "private/get-deposit-history", {"params": params}
            )
            or {}
        )
        return [
            Deposit.create_from_api(trx)
            for trx in data.get("deposit_list") or []
        ]

    async def get_withdrawal_history(
        self,
        coin: Coin,
        start_ts: int = None,
        end_ts: int = None,
        status: WithdrawalStatus = None,
        page: int = 0,
        page_size: int = 20,
    ) -> List[Withdrawal]:
        """Return all history for withdrawal transactions."""
        params = {"page_size": page_size, "page": page}
        if coin:
            params["currency"] = coin.name
        if start_ts:
            params["start_ts"] = int(start_ts) * 1000
        if end_ts:
            params["end_ts"] = int(end_ts) * 1000
        if status:
            params["status"] = status

        data = (
            await self.api.post(
                "private/get-withdrawal-history", {"params": params}
            )
            or {}
        )
        return [
            Withdrawal.create_from_api(trx)
            for trx in data.get("withdrawal_list") or []
        ]

    async def get_interest_history(
        self,
        coin: Coin,
        start_ts: int = None,
        end_ts: int = None,
        page: int = 0,
        page_size: int = 20,
    ) -> List[Interest]:
        """Return all history interest."""
        params = {"page_size": page_size, "page": page}
        if coin:
            params["currency"] = coin.name
        if start_ts:
            params["start_ts"] = int(start_ts) * 1000
        if end_ts:
            params["end_ts"] = int(end_ts) * 1000

        data = (
            await self.api.post(
                "private/margin/get-order-history", {"params": params}
            )
            or {}
        )
        return [
            Interest.create_from_api(interest)
            for interest in data.get("list") or []
        ]

    async def get_orders_history(
        self,
        pair: Pair = None,
        start_ts: int = None,
        end_ts: int = None,
        page: int = 0,
        page_size: int = 200,
    ) -> List[Order]:
        """Return all history orders."""
        params = {"page_size": page_size, "page": page}
        if pair:
            params["instrument_name"] = pair.name
        if start_ts:
            params["start_ts"] = int(start_ts) * 1000
        if end_ts:
            params["end_ts"] = int(end_ts) * 1000

        data = (
            await self.api.post(
                "private/get-order-history", {"params": params}
            )
            or {}
        )
        return [
            Order.create_from_api(self.pairs[order["instrument_name"]], order)
            for order in data.get("order_list") or []
        ]

    async def get_open_orders(
        self, pair: Pair = None, page: int = 0, page_size: int = 200
    ) -> List[Order]:
        """Return open orders."""
        params = {"page_size": page_size, "page": page}
        if pair:
            params["instrument_name"] = pair.name
        data = await self.api.post(
            "private/get-open-orders", {"params": params}
        )
        return [
            Order.create_from_api(self.pairs[order["instrument_name"]], order)
            for order in data.get("order_list") or []
        ]

    async def get_trades(
        self,
        pair: Pair = None,
        start_ts: int = None,
        end_ts: int = None,
        page: int = 0,
        page_size: int = 200,
    ) -> List[PrivateTrade]:
        """Return trades."""
        params = {"page_size": page_size, "page": page}
        if pair:
            params["instrument_name"] = pair.name
        if start_ts:
            params["start_ts"] = int(start_ts) * 1000
        if end_ts:
            params["end_ts"] = int(end_ts) * 1000
        data = await self.api.post("private/get-trades", {"params": params})
        return [
            PrivateTrade.create_from_api(
                self.pairs[trade["instrument_name"]], trade
            )
            for trade in data.get("trade_list") or []
        ]

    async def create_order(
        self,
        pair: Pair,
        side: OrderSide,
        type_: OrderType,
        quantity: float,
        price: float = 0,
        force_type: OrderForceType = OrderForceType.GOOD_TILL_CANCEL,
        exec_type: OrderExecType = OrderExecType.MARKET,
        client_id: int = None,
    ) -> int:
        """Create raw order with buy or sell side."""
        data = {
            "instrument_name": pair.name,
            "side": side.value,
            "type": type_.value,
        }
        data["time_in_force"] = force_type.value
        data["exec_inst"] = exec_type.value

        quantity = "{:.{}f}".format(quantity, pair.quantity_precision)
        if type_ == OrderType.MARKET and side == OrderSide.BUY:
            data["notional"] = quantity
        else:
            data["quantity"] = quantity

        if client_id:
            data["client_oid"] = str(client_id)

        if price:
            if type_ == OrderType.MARKET:
                raise ValueError(
                    "Error, MARKET execution do not support price value"
                )
            data["price"] = "{:.{}f}".format(price, pair.price_precision)

        resp = await self.api.post("private/create-order", {"params": data})
        return int(resp["order_id"])

    async def buy_limit(
        self,
        pair: Pair,
        quantity: float,
        price: float,
        force_type: OrderForceType = OrderForceType.GOOD_TILL_CANCEL,
        exec_type: OrderExecType = OrderExecType.MARKET,
        client_id: int = None,
    ) -> int:
        """Buy limit order."""
        return await self.create_order(
            pair,
            OrderSide.BUY,
            OrderType.LIMIT,
            quantity,
            price,
            force_type,
            exec_type,
        )

    async def sell_limit(
        self,
        pair: Pair,
        quantity: float,
        price: float,
        force_type: OrderForceType = OrderForceType.GOOD_TILL_CANCEL,
        exec_type: OrderExecType = OrderExecType.MARKET,
        client_id: int = None,
    ) -> int:
        """Sell limit order."""
        return await self.create_order(
            pair,
            OrderSide.SELL,
            OrderType.LIMIT,
            quantity,
            price,
            force_type,
            exec_type,
        )

    async def wait_for_status(
        self, order_id: int, statuses, delay: int = 0.1
    ) -> None:
        """Wait for order status."""
        order = await self.get_order(order_id)

        for _ in range(self.api.retries):
            if order.status in statuses:
                break

            await asyncio.sleep(delay)
            order = await self.get_order(order_id)

        if order.status not in statuses:
            raise ApiError(
                f"Status not changed for: {order}, must be in: {statuses}"
            )

    async def buy_market(
        self, pair: Pair, spend: float, wait_for_fill=False
    ) -> int:
        """Buy market order."""
        order_id = await self.create_order(
            pair, OrderSide.BUY, OrderType.MARKET, spend
        )
        if wait_for_fill:
            await self.wait_for_status(
                order_id,
                (
                    OrderStatus.FILLED,
                    OrderStatus.CANCELED,
                    OrderStatus.EXPIRED,
                    OrderStatus.REJECTED,
                ),
            )

        return order_id

    async def sell_market(
        self, pair: Pair, quantity: float, wait_for_fill=False
    ) -> int:
        """Sell market order."""
        order_id = await self.create_order(
            pair, OrderSide.SELL, OrderType.MARKET, quantity
        )

        if wait_for_fill:
            await self.wait_for_status(
                order_id,
                (
                    OrderStatus.FILLED,
                    OrderStatus.CANCELED,
                    OrderStatus.EXPIRED,
                    OrderStatus.REJECTED,
                ),
            )

        return order_id

    async def get_order(self, order_id: int) -> Order:
        """Get order info."""
        order_info = {}
        retries = 0

        while True:
            data = await self.api.post(
                "private/get-order-detail",
                {"params": {"order_id": str(order_id)}},
            )
            order_info = data.get("order_info", {})
            if not order_info:
                if retries < 10:
                    await asyncio.sleep(0.5)
                    retries += 1
                else:
                    raise ApiError(f"No order info found for id: {order_id}")
            else:
                break

        return Order.create_from_api(
            self.pairs[order_info["instrument_name"]],
            order_info,
            data["trade_list"],
        )

    async def cancel_order(
        self, order_id: int, pair: Pair, check_status=False
    ) -> None:
        """Cancel order."""
        await self.api.post(
            "private/cancel-order",
            {"params": {"order_id": order_id, "instrument_name": pair.name}},
        )

        if not check_status:
            return

        await self.wait_for_status(
            order_id,
            (OrderStatus.CANCELED, OrderStatus.EXPIRED, OrderStatus.REJECTED),
        )

    async def cancel_open_orders(self, pair: Pair) -> None:
        """Cancel all open orders."""
        await self.api.post(
            "private/cancel-all-orders",
            {"params": {"instrument_name": pair.name}},
        )

    async def listen_balance(self) -> Balance:
        async for data in self.api.listen("user", "user.balance", sign=True):
            for bal in data.get("data", []):
                yield Balance(
                    total=bal["balance"],
                    available=bal["available"],
                    in_orders=bal["order"],
                    in_stake=bal["stake"],
                    coin=Coin(bal["currency"]),
                )

    async def listen_orders(self, pair: Pair) -> Order:
        async for data in self.api.listen(
            "user", f"user.order.{pair.name}", sign=True
        ):
            for order in data.get("data", []):
                yield Order.create_from_api(
                    self.pairs[data["instrument_name"]], order
                )



from .structs import Coin

AAVE = Coin("AAVE")
ACA = Coin("ACA")
ADA = Coin("ADA")
AERGO = Coin("AERGO")
AGLD = Coin("AGLD")
AKT = Coin("AKT")
ALGO = Coin("ALGO")
ALI = Coin("ALI")
ALICE = Coin("ALICE")
ALPHA = Coin("ALPHA")
AMP = Coin("AMP")
ANC = Coin("ANC")
ANKR = Coin("ANKR")
ANY = Coin("ANY")
APE = Coin("APE")
API3 = Coin("API3")
AR = Coin("AR")
ARPA = Coin("ARPA")
ATOM = Coin("ATOM")
AUDIO = Coin("AUDIO")
AURORA = Coin("AURORA")
AUTO = Coin("AUTO")
AVAX = Coin("AVAX")
AXS = Coin("AXS")
BADGER = Coin("BADGER")
BAL = Coin("BAL")
BAND = Coin("BAND")
BAT = Coin("BAT")
BCH = Coin("BCH")
BICO = Coin("BICO")
BIFI = Coin("BIFI")
BNT = Coin("BNT")
BOBA = Coin("BOBA")
BOND = Coin("BOND")
BOSON = Coin("BOSON")
BRZ = Coin("BRZ")
BTC = Coin("BTC")
BUSD = Coin("BUSD")
C98 = Coin("C98")
CELR = Coin("CELR")
CHR = Coin("CHR")
CHZ = Coin("CHZ")
CKB = Coin("CKB")
COMP = Coin("COMP")
COTI = Coin("COTI")
CQT = Coin("CQT")
CRO = Coin("CRO")
CRPT = Coin("CRPT")
CRV = Coin("CRV")
CSPR = Coin("CSPR")
CTSI = Coin("CTSI")
CVX = Coin("CVX")
DAI = Coin("DAI")
DAR = Coin("DAR")
DERC = Coin("DERC")
DGB = Coin("DGB")
DIA = Coin("DIA")
DINO = Coin("DINO")
DOGE = Coin("DOGE")
DOT = Coin("DOT")
DYDX = Coin("DYDX")
EFI = Coin("EFI")
EGLD = Coin("EGLD")
ELON = Coin("ELON")
ENJ = Coin("ENJ")
ENS = Coin("ENS")
EOS = Coin("EOS")
EPS = Coin("EPS")
ERN = Coin("ERN")
ETC = Coin("ETC")
ETH = Coin("ETH")
FARM = Coin("FARM")
FET = Coin("FET")
FIL = Coin("FIL")
FITFI = Coin("FITFI")
FLOW = Coin("FLOW")
FLUX = Coin("FLUX")
FORTH = Coin("FORTH")
FTM = Coin("FTM")
FXS = Coin("FXS")
GALA = Coin("GALA")
GHST = Coin("GHST")
GLM = Coin("GLM")
GLMR = Coin("GLMR")
GMT = Coin("GMT")
GODS = Coin("GODS")
GRT = Coin("GRT")
GTC = Coin("GTC")
GUSD = Coin("GUSD")
HBAR = Coin("HBAR")
HERO = Coin("HERO")
HIGH = Coin("HIGH")
HNT = Coin("HNT")
HOD = Coin("HOD")
HOT = Coin("HOT")
HUSD = Coin("HUSD")
ICP = Coin("ICP")
ICX = Coin("ICX")
ILV = Coin("ILV")
IMX = Coin("IMX")
INJ = Coin("INJ")
IOTX = Coin("IOTX")
IQ = Coin("IQ")
JASMY = Coin("JASMY")
JOE = Coin("JOE")
KAVA = Coin("KAVA")
KEEP = Coin("KEEP")
KLAY = Coin("KLAY")
KNC = Coin("KNC")
KP3R = Coin("KP3R")
KRL = Coin("KRL")
KSM = Coin("KSM")
LDO = Coin("LDO")
LINK = Coin("LINK")
LPT = Coin("LPT")
LRC = Coin("LRC")
LSK = Coin("LSK")
LTC = Coin("LTC")
LUNA = Coin("LUNA")
LUNC = Coin("LUNC")
MANA = Coin("MANA")
MASK = Coin("MASK")
MATIC = Coin("MATIC")
MBL = Coin("MBL")
MC = Coin("MC")
MCO = Coin("MCO")
MDT = Coin("MDT")
MIR = Coin("MIR")
MKR = Coin("MKR")
MLN = Coin("MLN")
MMF = Coin("MMF")
MOVR = Coin("MOVR")
MULTI = Coin("MULTI")
MXC = Coin("MXC")
NANO = Coin("NANO")
NEAR = Coin("NEAR")
NEO = Coin("NEO")
NKN = Coin("NKN")
NMR = Coin("NMR")
NU = Coin("NU")
OCEAN = Coin("OCEAN")
OGN = Coin("OGN")
OMG = Coin("OMG")
ONE = Coin("ONE")
ONEINCH = Coin("1INCH")
ONT = Coin("ONT")
OXT = Coin("OXT")
PAX = Coin("PAX")
PAXG = Coin("PAXG")
PENDLE = Coin("PENDLE")
PERP = Coin("PERP")
PLA = Coin("PLA")
POLS = Coin("POLS")
POLY = Coin("POLY")
POWR = Coin("POWR")
PRQ = Coin("PRQ")
PYR = Coin("PYR")
QI = Coin("QI")
QNT = Coin("QNT")
QRDO = Coin("QRDO")
QTUM = Coin("QTUM")
QUICK = Coin("QUICK")
RAD = Coin("RAD")
RADAR = Coin("RADAR")
RARE = Coin("RARE")
RARI = Coin("RARI")
RAY = Coin("RAY")
REEF = Coin("REEF")
REN = Coin("REN")
REQ = Coin("REQ")
RGT = Coin("RGT")
RLC = Coin("RLC")
RLY = Coin("RLY")
RNDR = Coin("RNDR")
RSR = Coin("RSR")
RUNE = Coin("RUNE")
RVN = Coin("RVN")
SAND = Coin("SAND")
SC = Coin("SC")
SDN = Coin("SDN")
SGB = Coin("SGB")
SHIB = Coin("SHIB")
SKL = Coin("SKL")
SLP = Coin("SLP")
SNT = Coin("SNT")
SNX = Coin("SNX")
SOL = Coin("SOL")
SPELL = Coin("SPELL")
SPS = Coin("SPS")
SRM = Coin("SRM")
STORJ = Coin("STORJ")
STRAX = Coin("STRAX")
STX = Coin("STX")
SUPER = Coin("SUPER")
SUSHI = Coin("SUSHI")
T = Coin("T")
TFUEL = Coin("TFUEL")
THETA = Coin("THETA")
TONIC = Coin("TONIC")
TRB = Coin("TRB")
TRIBE = Coin("TRIBE")
TRU = Coin("TRU")
TUSD = Coin("TUSD")
UMA = Coin("UMA")
UNI = Coin("UNI")
USDC = Coin("USDC")
USDP = Coin("USDP")
USDT = Coin("USDT")
VET = Coin("VET")
VOXEL = Coin("VOXEL")
VRA = Coin("VRA")
VTHO = Coin("VTHO")
VVS = Coin("VVS")
WAVE = Coin("WAVE")
WAVES = Coin("WAVES")
WAXP = Coin("WAXP")
WBTC = Coin("WBTC")
WEMIX = Coin("WEMIX")
WOO = Coin("WOO")
XLM = Coin("XLM")
XRP = Coin("XRP")
XTZ = Coin("XTZ")
XYO = Coin("XYO")
YFI = Coin("YFI")
YGG = Coin("YGG")
ZBC = Coin("ZBC")
ZIL = Coin("ZIL")
ZRX = Coin("ZRX")


def all():
    return [
        value for name, value in globals().items() if isinstance(value, Coin)
    ]



import time
from dataclasses import dataclass
from datetime import datetime
from enum import Enum, IntEnum
from typing import Dict, List

from cached_property import cached_property

from .helpers import round_down, round_up


@dataclass
class Coin:
    exchange_name: str

    @property
    def name(self):
        return self.exchange_name.replace("1", "ONE")

    def __hash__(self):
        return self.name.__hash__()


@dataclass
class Pair:
    exchange_name: str
    price_precision: int
    quantity_precision: int

    @property
    def name(self):
        return self.exchange_name.replace("1", "ONE")

    @cached_property
    def base_coin(self) -> Coin:
        return Coin(self.name.split("_")[0])

    @cached_property
    def quote_coin(self) -> Coin:
        return Coin(self.name.split("_")[1])

    def round_price(self, price):
        return round_down(price, self.price_precision)

    def round_quantity(self, quantity):
        return round_down(quantity, self.quantity_precision)

    def __hash__(self):
        return self.name.__hash__()


@dataclass
class MarketTicker:
    pair: Pair
    buy_price: float
    sell_price: float
    trade_price: float
    time: int
    volume: float
    high: float
    low: float
    change: float

    @classmethod
    def from_api(cls, pair, data):
        return cls(
            pair=pair,
            buy_price=pair.round_price(data["b"]),
            sell_price=pair.round_price(data["k"]),
            trade_price=pair.round_price(data["a"]),
            time=int(data["t"] / 1000),
            volume=pair.round_quantity(data["v"]),
            high=pair.round_price(data["h"]),
            low=pair.round_price(data["l"]),
            change=round_down(data["c"], 3),
        )


class OrderSide(str, Enum):
    BUY = "BUY"
    SELL = "SELL"


@dataclass
class MarketTrade:
    id: int
    time: int
    price: float
    quantity: float
    side: OrderSide
    pair: Pair

    @classmethod
    def from_api(cls, pair: Pair, data: Dict):
        return cls(
            id=data["d"],
            time=int(data["t"] / 1000),
            price=pair.round_price(data["p"]),
            quantity=pair.round_quantity(data["q"]),
            side=OrderSide(data["s"].upper()),
            pair=pair,
        )


class Period(str, Enum):
    MINS = "1m"
    MINS_5 = "5m"
    MINS_15 = "15m"
    MINS_30 = "30m"
    HOURS = "1h"
    HOURS_4 = "4h"
    HOURS_6 = "6h"
    HOURS_12 = "12h"
    DAY = "1D"
    WEEK = "7D"
    WEEK_2 = "14D"
    MONTH_1 = "1M"


@dataclass
class Candle:
    time: int
    open: float
    high: float
    low: float
    close: float
    volume: float
    pair: Pair

    @classmethod
    def from_api(cls, pair: Pair, data: Dict):
        return cls(
            time=int(data["t"] / 1000),
            open=pair.round_price(data["o"]),
            high=pair.round_price(data["h"]),
            low=pair.round_price(data["l"]),
            close=pair.round_price(data["c"]),
            volume=pair.round_quantity(data["v"]),
            pair=pair,
        )


@dataclass
class OrderInBook:
    price: float
    quantity: float
    count: int
    pair: Pair
    side: OrderSide

    @property
    def volume(self) -> float:
        return self.pair.round_quantity(self.price * self.quantity)


@dataclass
class OrderBook:
    buys: List[OrderInBook]
    sells: List[OrderInBook]
    pair: Pair

    @property
    def spread(self) -> float:
        return round_down(self.sells[-1].price / self.buys[0].price - 1, 6)


@dataclass
class Balance:
    total: float
    available: float
    in_orders: float
    in_stake: float
    coin: Coin

    @classmethod
    def from_api(cls, data):
        return cls(
            total=data["balance"],
            available=data["available"],
            in_orders=data["order"],
            in_stake=data["stake"],
            coin=Coin(data["currency"]),
        )


class OrderType(str, Enum):
    LIMIT = "LIMIT"
    MARKET = "MARKET"
    STOP_LOSS = "STOP_LOSS"
    STOP_LIMIT = "STOP_LIMIT"
    TAKE_PROFIT = "TAKE_PROFIT"
    TAKE_PROFIT_LIMIT = "TAKE_PROFIT_LIMIT"


class OrderStatus(str, Enum):
    ACTIVE = "ACTIVE"
    FILLED = "FILLED"
    CANCELED = "CANCELED"
    REJECTED = "REJECTED"
    EXPIRED = "EXPIRED"
    PENDING = "PENDING"


class OrderExecType(str, Enum):
    MARKET = ""
    POST_ONLY = "POST_ONLY"


class OrderForceType(str, Enum):
    GOOD_TILL_CANCEL = "GOOD_TILL_CANCEL"
    FILL_OR_KILL = "FILL_OR_KILL"
    IMMEDIATE_OR_CANCEL = "IMMEDIATE_OR_CANCEL"


@dataclass
class PrivateTrade:
    id: int
    side: OrderSide
    pair: Pair
    fees: float
    fees_coin: Coin
    created_at: int
    filled_price: float
    filled_quantity: float
    order_id: int

    @cached_property
    def is_buy(self):
        return self.side == OrderSide.BUY

    @cached_property
    def is_sell(self):
        return self.side == OrderSide.SELL

    @classmethod
    def create_from_api(cls, pair: Pair, data: Dict) -> "PrivateTrade":
        return cls(
            id=int(data["trade_id"]),
            side=OrderSide(data["side"]),
            pair=pair,
            fees=round_up(data["fee"], 8),
            fees_coin=Coin(data["fee_currency"]),
            created_at=int(data["create_time"] / 1000),
            filled_price=pair.round_price(data["traded_price"]),
            filled_quantity=pair.round_quantity(data["traded_quantity"]),
            order_id=int(data["order_id"]),
        )


@dataclass
class Order:
    id: int
    status: OrderStatus
    side: OrderSide
    price: float
    quantity: float
    client_id: str
    created_at: int
    updated_at: int
    type: OrderType
    pair: Pair
    filled_quantity: float
    filled_price: float
    fees_coin: Coin
    force_type: OrderForceType
    trigger_price: float
    trades: List[PrivateTrade]

    @cached_property
    def is_buy(self):
        return self.side == OrderSide.BUY

    @cached_property
    def is_sell(self):
        return self.side == OrderSide.SELL

    @cached_property
    def is_active(self):
        return self.status == OrderStatus.ACTIVE

    @cached_property
    def is_canceled(self):
        return self.status == OrderStatus.CANCELED

    @cached_property
    def is_rejected(self):
        return self.status == OrderStatus.REJECTED

    @cached_property
    def is_expired(self):
        return self.status == OrderStatus.EXPIRED

    @cached_property
    def is_filled(self):
        return self.status == OrderStatus.FILLED

    @cached_property
    def is_pending(self):
        return self.status == OrderStatus.PENDING

    @cached_property
    def volume(self):
        return self.pair.round_quantity(self.price * self.quantity)

    @cached_property
    def filled_volume(self):
        return self.pair.round_quantity(
            self.filled_price * self.filled_quantity
        )

    @cached_property
    def remain_volume(self):
        return self.pair.round_quantity(self.filled_volume - self.volume)

    @cached_property
    def remain_quantity(self):
        return self.pair.round_quantity(self.quantity - self.filled_quantity)

    @classmethod
    def create_from_api(
        cls, pair: Pair, data: Dict, trades: List[Dict] = None
    ) -> "Order":
        fees_coin, trigger_price = None, None
        if data["fee_currency"]:
            fees_coin = Coin(data["fee_currency"])
        if data.get("trigger_price") is not None:
            trigger_price = pair.round_price(data["trigger_price"])

        trades = [
            PrivateTrade.create_from_api(pair, trade) for trade in trades or []
        ]

        return cls(
            id=int(data["order_id"]),
            status=OrderStatus(data["status"]),
            side=OrderSide(data["side"]),
            price=pair.round_price(data["avg_price"] or data["price"]),
            quantity=pair.round_quantity(data["quantity"]),
            client_id=data["client_oid"],
            created_at=int(data["create_time"] / 1000),
            updated_at=int(data["update_time"] / 1000),
            type=OrderType(data["type"]),
            pair=pair,
            filled_price=pair.round_price(data["avg_price"]),
            filled_quantity=pair.round_quantity(data["cumulative_quantity"]),
            fees_coin=fees_coin,
            force_type=OrderForceType(data["time_in_force"]),
            trigger_price=trigger_price,
            trades=trades,
        )


@dataclass
class Interest:
    loan_id: int
    coin: Coin
    interest: float
    stake_amount: float
    interest_rate: float

    @classmethod
    def create_from_api(cls, data: Dict) -> "Interest":
        return cls(
            loan_id=int(data["loan_id"]),
            coin=Coin(data["currency"]),
            interest=float(data["interest"]),
            stake_amount=float(data["stake_amount"]),
            interest_rate=float(data["interest_rate"]),
        )


class WithdrawalStatus(str, Enum):
    PENDING = "0"
    PROCESSING = "1"
    REJECTED = "2"
    PAYMENT_IN_PROGRESS = "3"
    PAYMENT_FAILED = "4"
    COMPLETED = "5"
    CANCELLED = "6"


class DepositStatus(str, Enum):
    NOT_ARRIVED = "0"
    ARRIVED = "1"
    FAILED = "2"
    PENDING = "3"


class TransactionType(IntEnum):
    WITHDRAWAL = 0
    DEPOSIT = 1


@dataclass
class Transaction:
    coin: Coin
    fee: float
    create_time: int
    id: str
    update_time: int
    amount: float
    address: str

    @staticmethod
    def _prepare(data):
        return dict(
            id=data["id"],
            coin=Coin(data["currency"]),
            fee=float(data["fee"]),
            create_time=datetime.fromtimestamp(
                int(data["create_time"]) / 1000
            ),
            update_time=datetime.fromtimestamp(
                int(data["update_time"]) / 1000
            ),
            amount=float(data["amount"]),
            address=data["address"],
        )


@dataclass
class Deposit(Transaction):
    status: DepositStatus

    @classmethod
    def create_from_api(cls, data: Dict) -> "Deposit":
        params = cls._prepare(data)
        params["status"] = DepositStatus(data["status"])
        return cls(**params)


@dataclass
class Withdrawal(Transaction):
    client_wid: str
    status: WithdrawalStatus
    txid: str

    @classmethod
    def create_from_api(cls, data: Dict) -> "Withdrawal":
        params = cls._prepare(data)
        params["client_wid"] = data.get("client_wid", "")
        params["status"] = WithdrawalStatus(data["status"])
        params["txid"] = data["txid"]
        return cls(**params)


class Timeframe(IntEnum):
    NOW = 0
    MINUTES = 60
    HOURS = 60 * MINUTES
    DAYS = 24 * HOURS
    WEEKS = 7 * DAYS
    MONTHS = 30 * DAYS

    @classmethod
    def resolve(cls, seconds: int) -> int:
        return seconds + int(time.time())
